#SkIndigo Ruby code is copyright Glare Technologies Limited, 2014, all rights reserved
#-----------------------------------------------------------------------------
#

def reinterpret_as_utf8(str)
	if String.method_defined?(:encode)
		str = str.encode('UTF-16', 'UTF-8', :invalid => :replace, :replace => '') # Convert to UTF-16
		str = str.encode('UTF-8', 'UTF-16') # Convert back to UTF-8
	end
	return str
end

skindigo_path = File.dirname(reinterpret_as_utf8(__FILE__))
load File.join(skindigo_path, "IndigoScatterEditor.rb")
load File.join(skindigo_path, "IndigoScatter.rb")


class IndigoScattersEditor

	def initialize() 

		@dlg = UI::WebDialog.new("SkIndigo Scatters Editor", true, "SkIndigoScattersEditor", 630,350,419,243, true) #width,height,left,top
		@dlg.set_file(File.join(File.dirname(reinterpret_as_utf8(__FILE__)), "ScattersEditor.html"))
		color = @dlg.get_default_dialog_color
		@dlg.set_background_color(color)
		
		@dlg.add_action_callback("editScatter") {|d,p|
			pair = p.split("=")
			id = pair[0]
			value = pair[1]
			
			on_edit_scatter(value.to_i)
		}
		
		@dlg.add_action_callback("deleteScatter") {|d,p|
			pair = p.split("=")
			id = pair[0]
			value = pair[1]
			
			on_delete_scatter(value.to_i)
		}
		
		@dlg.add_action_callback("loaded") {|d,p|
			on_loaded
		}
		
		@dlg.add_action_callback("addNewScatter") {|d,p|
			on_addNewScatter
		}
		
		@dlg.add_action_callback("addPreset") {|d,p|
			on_addPreset
		}
		
		@scatter_editor = nil
		
		@scatter_setttings = IndigoScatterSettings.new
	end

	def show
		@dlg.show
	end
	
	def on_loaded()
	
		@dlg.execute_script("clearScatters()")
		
		index = 0
		@scatter_setttings.scatters.each do |scatter|

			cmd = "addScatter(" + index.to_s + ", \"#{SkIndigo.javascriptEscape(scatter.name)}\")"
			@dlg.execute_script(cmd)
		
			index += 1
		end
	end
	

	def setValue(id, value)
		SkIndigo.set_value(@dlg, id, value)
	end

	
	def on_edit_scatter(index)
		#puts "on_edit_scatter(), index=#{index}"
	
		the_scatter = @scatter_setttings.scatters[index.to_i]
	
		@scatter_editor = IndigoScatterEditor.new(@scatter_setttings, the_scatter, self)
		@scatter_editor.show
	end
	
	def on_delete_scatter(index)
		#puts "on_delete_scatter(), index=#{index}"
	
		@scatter_setttings.scatters.delete_at(index)
	
		@scatter_setttings.save_scatter_settings
		
		on_loaded() # Re-layout widget
	end
	
	def on_addNewScatter()
		#puts  "on_addNewScatter()"
	
		scatter = IndigoScatter.new
		@scatter_setttings.scatters << scatter
		
		@scatter_setttings.save_scatter_settings
		
		on_loaded() # Re-layout widget
	end
	
	def on_addPreset()

		presets = {
			"Grass" => {
				:objects => [
					{
						:path => "scatters/grass/grass_clump.skp",
						:weight => 1.0
					}
				],
				:density =>150.0,
				:evenness => 1.0,
				:scale_std_dev => 0.05,
				:height_std_dev => 0.0
			}#,
			#"SamsLawn" => {
			#	:objects => [
			#		{
			#			:path => "scatters/Sams_lawn/Scatter_SamsPlaintainWeed.skp",
			#			:weight => 1.0
			#		}
			#	],
			#	:density =>150.0,
			#	:evenness => 1.0,
			#	:scale_std_dev => 0.05,
			#	:height_std_dev => 0.0
			#}
		}
	
		#Create the Preset Selection Dialog
		@preset_dlg = UI::WebDialog.new("Select Scatter Preset", 
			true, # scrollable
			"SkIndigoPresets", 300, 768, 272, 1, true)
		fn = File.join(File.dirname(reinterpret_as_utf8(__FILE__)), "ScatterPresets.html")
		@preset_dlg.set_file fn
		color = @preset_dlg.get_default_dialog_color 
		@preset_dlg.set_background_color(color)
		@preset_dlg.add_action_callback("assign_preset") {|d,p|
			params = p.split("=")
			key = params[0]
			value = params[1]
			
			if key == "id"
				preset = presets[value]
				#puts "preset: #{preset.inspect}"
				if preset
					scatter = IndigoScatter.new
					scatter.density = preset[:density]
					scatter.evenness = preset[:evenness]
					scatter.scale_std_dev = preset[:scale_std_dev]
					scatter.height_std_dev = preset[:height_std_dev]
					
					# Load components
					#TEMP:
					preset[:objects].each do |ob|
					
						model = Sketchup.active_model
						definitions = model.definitions
						
						path = File.join(File.dirname(reinterpret_as_utf8(__FILE__)), ob[:path])
						#puts "path: '#{path}'"
						begin
							componentdefinition = definitions.load path #"C:\\programming\\skindigo_trunk\\scenes\\SketchUp_8\\" + ob[:path]
						rescue 
							puts "Failed to load file '#{path}': #{$!}"
						end
						#componentdefinition = definitions.load_from_url "http://indigorenderer.com/montecarlo/cylinder.skp"
						#puts "componentdefinition: #{componentdefinition.inspect}"
						
						group = ProtoObjectGroup.new
						group.prototype_def_name = componentdefinition.name
						group.weight = ob[:weight]
						scatter.groups << group
					end
		
					# Add the new scatter to the scatter list
					@scatter_setttings.scatters << scatter
					@scatter_setttings.save_scatter_settings
				end
			end
			
			#loadValuesFromSU()   #updates the material editor with new material values
			#setVisibility()
			
			
			
			@preset_dlg.close()
			
			on_loaded() # Re-layout widget
			
			# Pop up edit dialog for new scatter
			on_edit_scatter(@scatter_setttings.scatters.length - 1)
		}
			
		@preset_dlg.show{}
	end
	
	def scatterChanged()
		on_loaded() # Re-layout widget
	end
end

